"""
zTrix Folder, a class to hold objects that connect with Asterisk.
Currently requires MXM simple product classes to work.
http://www.zope.org/Members/maxm/HowTo/easyProduct
(c) 2004 Salim Fadhley, Deskforce Ltd
http://www.deskforce.ltd.uk
http://wiki.stodge.org

$LastChangedDate: 2004-10-04 22:29:27 +0100 (Mon, 04 Oct 2004) $
$Rev: 17 $
$HeadURL: http://svn.stodge.org/ztrixProduct/trunk/ztrixProduct/ztrixFolder.py $

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

__author__="Salim Fadhley <sal@deskforce.ltd.uk>"
__id__="$Id: ztrixFolder.py 17 2004-10-04 21:29:27Z sal $"

# A handy module for making zope classes simpler to write.
from mxm import mxmObjectManager

# Zope Tabs
from App.Management import Tabs

# Utility
import sys, time, socket

# Asterisk related
from Asterisk.Manager import CoreManager, Manager
from Asterisk.Manager import BaseChannel
import Asterisk.Manager, Asterisk.Util
import Asterisk.Config

# Exceptions
from Asterisk.Manager import ActionFailed

# Publishing related
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.PythonScripts.PythonScript import PythonScript

# Security related
from AccessControl import getSecurityManager
from AccessControl.PermissionRole import PermissionRole
from Permissions import ManageAsteriskPermission, ManageAsteriskDefaultRoles
from AccessControl import ClassSecurityInfo
	

# We need to monkeypatch dw's class...
class ztrixManager(Manager):
	"""
	A crudely hacked version of DW's Manager class that exposes the _get_channel() method.
	"""
	def getChannel(self,channel_id):
		return self._get_channel(channel_id)

class ztrixFolder(mxmObjectManager.mxmObjectManager):
	"""
	A folderish object that lets you manage an Asterisk PBX.
		
	A Zope wrapper for the "Asterisk Manager and Channel objects" originally by David M. Wilson <dw-py-asterisk-Manager.py@botanicus.net>
	
	Also based on the MXM simple object-manager template, or more information see: http://www.zope.org/Members/maxm/HowTo/easyProduct
	"""
	__author__ = 'Salim Fadhley <sal@deskforce.ltd.uk>'
	__id__ = '$Id: ztrixFolder.py 17 2004-10-04 21:29:27Z sal $'
	
	meta_type = 'ztrixFolder'
	_mdict=globals()
	templates={}
	templates['status']=PageTemplateFile("zpt/status.zpt",_mdict)
	templates['console']=PageTemplateFile("zpt/console.zpt",_mdict)
	templates['commands']=PageTemplateFile("zpt/commands.zpt",_mdict)
	templates['channels']=PageTemplateFile("zpt/channels.zpt",_mdict)
	templates['channel']=PageTemplateFile("zpt/channel.zpt",_mdict)
	templates['extensions']=PageTemplateFile("zpt/extensions.zpt",_mdict)
	templates['licensing']=PageTemplateFile("zpt/licensing.zpt",_mdict)
	templates['extension']=PageTemplateFile("zpt/extension.zpt",_mdict)
	templates['contexts']=PageTemplateFile("zpt/contexts.zpt",_mdict)
	templates['context']=PageTemplateFile("zpt/context.zpt",_mdict)
	templates['originate']=PageTemplateFile("zpt/originate.zpt",_mdict)
	templates['iaxpeers']=PageTemplateFile("zpt/iaxpeers.zpt",_mdict)
	# 
	
	_properties = (	
		{'id':'title', 'type':'string', 'mode':'w'},
		{'id':'config_path', 'type':'string', 'mode':'w'}
		) 
	
	ManageAsteriskRoles = PermissionRole(ManageAsteriskPermission, ManageAsteriskDefaultRoles)
	
	security = ClassSecurityInfo()
	
	myManager = None
	
	wikiBase = "http://wiki.stodge.org/index.php/Ztrix"
	
	def filtered_manage_options(self, REQUEST=None):
        	"""
		Appends some ztrix object related tabs to the tab-bar. This most likely not a smart way to append items to the tab-bar, it works by overriding the method provided by the Tabs module. The advantage of this is it seems to work well with DocFinderTab, which is widely criticised for being Evil, but I quite like.
		"""
        	tabs = mxmObjectManager.mxmObjectManager.filtered_manage_options(self,REQUEST)
        	secman = getSecurityManager()
        	if len(tabs):
			# and secman.checkPermission(View, self.this())
			tabs.append({'label': 'Console', 'action': 'manage_console', 'help': ('ztrixProduct', 'Console.stx')})
			tabs.append({'label': 'Status', 'action': 'manage_status_form', 'help': ('ztrixProduct', 'Status.stx')})
			tabs.append({'label': 'API', 'action': 'manage_commands', 'help': ('ztrixProduct', 'Commands.stx')})
			tabs.append({'label': 'Orig', 'action': 'manage_originate', 'help': ('ztrixProduct', 'Originate.stx')})
			tabs.append({'label': 'Contexts', 'action': 'manage_contexts', 'help': ('ztrixProduct', 'Contexts.stx')})
			tabs.append({'label': 'Exten', 'action': 'manage_extensions', 'help': ('ztrixProduct', 'Extensions.stx')})
			tabs.append({'label': 'IAX2', 'action': 'manage_iaxpeers', 'help': ('ztrixProduct', 'Extensions.stx')})
		return tabs
	
	def __getstate__(self):
        	"""
		Ensures that non-persistable components are reset back to a form that is pickling-sane
		"""
		self.myManager.close()
		self.myManager=None
        	return dict(self.__dict__)
	
	def __init__(self, id):
		"""
		Inits the product with default values
		"""
        	self.id = id
        	self._defaults()
			
	def manage_status_form(self, REQUEST):
		"""
		A form to provide some basic status information.
		"""
		return self.templates['status'].__of__(self)()
	
	def manage_iaxpeers(self, REQUEST):
		"""
		A form to list out IAX2 protocol peers.
		"""
		opts={}
		opts['iaxpeers']=self.IAXpeersDict()
		
		return self.templates['iaxpeers'].__of__(self)("foo",[],**opts)
	
	def manage_commands(self, REQUEST):
		"""
		A screen that lists out all the Asterix PBX's commands.
		"""
		return self.templates['commands'].__of__(self)()
	
	def manage_channels(self, REQUEST):
		"""
		A screen that lists out all Asterisk PBX channels.
		"""
		return self.templates['channels'].__of__(self)()
	
	def manage_channel(self, REQUEST):
		"""
		A screen that prints information about a single Asterisk Channel
		"""
		myChannel=None
		
		opts={}
		form=REQUEST.form
		
		if form.has_key('channel_id'):
			opts['channel_id']=form['channel_id']
			opts['channel']=self.ChannelStatus(form['channel_id'])
		
		return self.templates['channel'].__of__(self)("foo",[],**opts)
	
	def manage_extensions(self, REQUEST):
		"""
		A screen that lists out all Asterisk PBX extensions.
		"""
		return self.templates['extensions'].__of__(self)()
	
	def manage_extension(self, REQUEST):
		"""
		A screen that provides information about a particular extension.
		"""
		myExten=None
		opts={}
		form=REQUEST.form
		
		if form.has_key('extension_id'):
			opts['extension_id']=form['extension_id']
			opts['context_id']=form['context_id']
			opts['extension']=self.ExtensionStatusDict(opts['context_id'], opts['extension_id'])
			
		return self.templates['extension'].__of__(self)("bla",[],**opts)
	
	def manage_contexts(self, REQUEST):
		"""
		A screen that lists out all Asterisk PBX contexts.
		"""
		return self.templates['contexts'].__of__(self)()
	
	def manage_context(self, REQUEST):
		"""
		A screen that provides information about a particular context.
		"""
		return self.templates['context'].__of__(self)()
	
	def manage_originate(self, REQUEST):
		"""
		A screen that allows a manager to originate a call.
		"""
		opts={}
		form=REQUEST.form
		
		opts['fields']=self.originate_fields()
		if form.has_key('channel'):
			opts['params']={}
			for key in opts['fields']:
				if len(form[key])==0:
					# Do nothing
					pass
				else:
					opts['params'][key]=form[key]
			
			# Try to originate
			try:
				opts['params']['async']=True
				opts['reply']=self.Manager().Originate( **opts['params'] )
			except ActionFailed, e:
				opts['error']=e
				
		return self.templates['originate'].__of__(self)(**opts)
			
			

	def manage_console(self, REQUEST):
		"""
		A form to provide some basic status information.
		"""
		myCommand=None
		opts={}
		form=REQUEST.form
		
		if form.has_key('command'):
			opts['command']=form['command']
			try:
				opts['reply']=self.Manager().Command(form['command'])
			except 	ValueError, myError:
					opts['error']="There was an error"

		return self.templates['console'].__of__(self)("foo",[1,2,3],**opts)	
			
	def ExtensionStatus(self, context_id, extension_id):
		"""
		Provide the status of an extension.
		"""
		return self.Manager().ExtensionState(context_id, extension_id)
	
	def ExtensionStatusDict(self, context_id, extension_id):
		"""
		Provide the status of an extension, in handy Dicts for ease of use in ZPT.
		"""
		return self._unravelDictlikeObject(self.ExtensionStatus(context_id, extension_id))
			
	def Channel(self, channel_id):
		"""
		Given an ID, this method returns a corresponding channel object.
		"""
		myManager=self.Manager()
		
		for channel in myManager.Status():
			if channel.id==channel_id:
				return channel
	
	def ChannelStatus(self, channel_id):
		"""
		Given a channel_id, this method returns a ZPT friendly status dict.
		"""
		myStatus=self.Status()
		for channel in myStatus:
			if channel.id == channel_id:
				return self.ChannelDict(channel, myStatus) 
		"""
		Need to fix, now that the Channel class now supports correct hashing.
		"""	
	
	def Manager(self):
		"""
		Connect to the Asterisk Manager if nececary and return an asterisk manager object.
		"""
		if type(self.myManager) == type(None):
			# Manager has not yet been intialised
			self.myManager = self.connectManager()
		return self.myManager
		
	
	def connectManager(self):
		"""
		Connect to the Asterisk server and return a regular Manager object as an object in the current context.
		"""
		return ztrixManager(*self.asterisk_config())
		
	def asterisk_config(self):
		"""
		Returns a tuple containing an asterisk configuration data. This can be used to init a Manager object.
		"""
		return Asterisk.Config.Config(getattr(self, 'config_path', None)).get_connection()
			
	def Status(self):
		"""
		Returns a dic of channel-object, chanal-param-dict pairs. When used TTW, it's better to call this via StatusDicts
		"""
		return self.Manager().Status()
	
	def StatusDicts(self):
		"""
		Return the status of your Asterisk PBX with no messy objects that are hard to work with TTW. 
		"""
		output=[]
		myChannels = self.Status()
		for channel in myChannels:
			output.append(self.ChannelDict(channel, myChannels))
		return output
			
	def ChannelDict(self, myChannel, myStatus):
		"""
		Given a channel object, returns a simplified dict of it's key values.
		"""
		channel_info=self._unravelDictlikeObject(myStatus[myChannel])
		channel_info['id']= myChannel.id
		return channel_info
	
	def ListCommands(self):
		"""
		Return an object listing of all the commands the Asterisk PBX Manager API is capable of executing.
		"""
		return self.Manager().ListCommands()
	
	def ListCommandsDict(self):	
		"""
		Return an object listing of all the commands the Asterisk PBX Manager API is capable of executing. Output is re-organised in a ZPT friendly dict,
		also the system tries to guess which of the Manager API classes each method belongs to.
		"""
		commands=self._unravelDictlikeObject(self.ListCommands())
		classes=[Manager, BaseChannel]
		
		output={}
		for command in commands:
			cmdinfo={}
			cmdinfo['desc']=commands[command]
			for myClass in classes:
				if hasattr(myClass, command):
					cmdinfo[myClass.__name__]=getattr(myClass,command).__doc__
				else:
					cmdinfo[myClass.__name__]=None
			output[command]=cmdinfo
		return output
		
	def IAXpeers(self):
		"""
		Does something with IAXpeers, not sure what yet.
		"""
		return self.Manager()
	
	def IAXpeersDict(self):
		"""
		Dunno yet
		"""
		return str(self.IAXpeers())
	
	def _unravelDictlikeObject(self, dictlike):
		"""
		Takes a funny dictlike object and turns it into a normal dict.
		"""
		output={}
		for key in dictlike.keys():
			output[key]=dictlike[key]
		return output
		
	def originate_fields(self):
		"""
		Return a dict of fields required to originate a call
		"""
		originate_fields={'channel':{'desc':"Channel to bridge with"}
			,'context':{'desc':"Dialplan context to bridge with"}
			,'extension':{'desc':"Context extension to bridge with"}
			,'priority':{'desc':"Context priority to bridge with"}
			,'application':{'desc':"Application to bridge with"}
			,'data':{'desc':"Application parameters"}
			,'timeout':{'desc':"Answer timeout for channel in milliseconds."}
			,'caller_id':{'desc':"Outgoing channel Caller ID"}
			,'variable':{'desc':"channel variable to set (K=V[|K2=V2[|..]])"}
			,'account':{'desc':"CDR account code."}
			}
		return originate_fields

	
def manage_addAction(self, id=None, REQUEST=None):
	"Add instance to parent ObjectManager"
	mxmObjectManager.addClass(self, id, ztrixFolder, REQUEST)

constructors =  (mxmObjectManager.manage_addForm, manage_addAction)

